import logging
from vesselasid.asid import Asid


class VesselAsidRenderer:
    def __init__(self, asid):
        self.asid = asid

    def start(self):
        return

    def stop(self):
        return

    def note_on(self, msg):
        return

    def note_off(self, msg):
        return

    def control_change(self, msg):
        return

    def pitchwheel(self, msg):
        return
